/*
 * Copyright (c) 2015 N. Billard.
 */
package com.massa.mrules.demos.employeetransportation;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.Collection;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.mrules.context.compile.CompilationLevel;
import com.massa.mrules.context.execute.MExecutionContext;
import com.massa.mrules.demos.employeetransportation.model.Employee;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.exception.MExecutionException;
import com.massa.mrules.factory.ser.SerializationFactory;
import com.massa.mrules.factory.xml.RichXMLFactory;
import com.massa.mrules.set.IMruleExecutionSet;

@RunWith(Parameterized.class)
public class EmployeeTransportationTest {
	private static final Log LOG = LogFactory.getLog(EmployeeTransportationTest.class.getName());

	final CompilationLevel level;
	final String xmlName;

	public EmployeeTransportationTest(final CompilationLevel level, final String xmlName) {
		this.level = level;
		this.xmlName = xmlName;
	}

	@Parameters
	public static Collection<Object[]> configs() {
		return Arrays.asList(new Object[][] {
			{CompilationLevel.STANDARD, EmployeeTransportationTest.XML1},
			{CompilationLevel.STANDARD, EmployeeTransportationTest.XML2},
			{CompilationLevel.STANDARD, EmployeeTransportationTest.XML3},
			{CompilationLevel.LIGHT, EmployeeTransportationTest.XML1},
			{CompilationLevel.LIGHT, EmployeeTransportationTest.XML2},
			{CompilationLevel.LIGHT, EmployeeTransportationTest.XML3}
		});
	}

	private static final String XML1 = "com/massa/mrules/demos/employeetransportation/rules_1.xml";
	private static final String XML2 = "com/massa/mrules/demos/employeetransportation/rules_2.xml";
	private static final String XML3 = "com/massa/mrules/demos/employeetransportation/rules_3.xml";

	String[][] inp = new String[][]{
		new String[]{ "PARIS", "METROPOLITAN", "25" },
		new String[]{ "PARIS", "METROPOLITAN", "120" },
		new String[]{ "PARIS", "PERSONAL CAR", "32" },
		new String[]{ "PARIS", "PERSONAL CAR", "84" },
		new String[]{ "PARIS", "PEDESTRIAN", "84" },

		new String[]{ "LONDON", "METROPOLITAN", "25" },
		new String[]{ "LONDON", "TRAMWAY", "120" },
		new String[]{ "LONDON", "BICYCLE", "32" },
		new String[]{ "LONDON", "PERSONAL CAR", "84" },
		new String[]{ "LONDON", "PEDESTRIAN", "84" },

		new String[]{ "PARIS", "BOAT", "84" },
		new String[]{ "ALMOSTPARIS", "PERSONAL CAR", "32" },
	};

	String[][] res = new String[][]{
		new String[]{ "true", "FLAT RATE", "30" },
		new String[]{ "true", "FLAT RATE", "50" },
		new String[]{ "true", "COMPENSATION", null },
		new String[]{ "true", "REAL", null },
		new String[]{ "false", null, null },

		new String[]{ null, null, null },
		new String[]{ "true", "FLAT RATE", "36" },
		new String[]{ "true", "FLAT RATE", "27.53" },
		new String[]{ "true", "REAL", null },
		new String[]{ "false", null, null },

		new String[]{ "true", "FLAT RATE", "126" },
		new String[]{ "true", "COMPENSATION", null },
	};



	@Test
	public void employeeTransportationDemo() throws MConfigurationException, MExecutionException, IOException {
		final RichXMLFactory factory = new RichXMLFactory(this.level);
		final Employee employee = new Employee();

		final InputStream xml = ClassLoader.getSystemResourceAsStream(this.xmlName);
		final IMruleExecutionSet eng = factory.read(xml);

		@SuppressWarnings("unchecked")
		final
		MExecutionContext<Employee> ctxt = (MExecutionContext<Employee>) eng.getContextFactory().getExecutionContext();
		ctxt.setInput(employee);
		ctxt.execute();

		for (int idx = 0 ; idx < this.inp.length ; idx++) {
			this.reset(employee);
			employee.setTown(this.inp[idx][0]);
			employee.setTransportationMode(this.inp[idx][1]);
			employee.setTransportationTime(Integer.valueOf(this.inp[idx][2]));

			ctxt.execute();

			Assert.assertEquals("index: " + idx, this.res[idx][0] == null ? null : Boolean.valueOf(this.res[idx][0]), employee.getRepayment());
			Assert.assertEquals("index: " + idx, this.res[idx][1], employee.getRepaymentType());
			Assert.assertEquals("index: " + idx, this.res[idx][2] == null ? null : Double.valueOf(this.res[idx][2]), employee.getRepaymentAmount());
		}

		//Time check
		EmployeeTransportationTest.LOG.info("Beginning performance test.");
		ctxt.setLoggerName(".perf"); //to avoid debug and info.
		final long t1 = System.currentTimeMillis();
		for (int i = 0 ; i < 10000 ; i++) {
			for (int idx = 0 ; idx < this.inp.length ; idx++) {
				this.reset(employee);
				employee.setTown(this.inp[idx][0]);
				employee.setTransportationMode(this.inp[idx][1]);
				employee.setTransportationTime(Integer.valueOf(this.inp[idx][2]));

				ctxt.execute();
			}
		}
		EmployeeTransportationTest.LOG.info("Time for 10000 tests : " + (System.currentTimeMillis() - t1));

		//Read / Write tests.
		EmployeeTransportationTest.LOG.info("Human readable version:\n" + eng.toString());

		final StringWriter w = new StringWriter();
		factory.write(w, eng);
		EmployeeTransportationTest.LOG.info(w.toString());

		final ByteArrayOutputStream o = new ByteArrayOutputStream();
		final SerializationFactory sfactory = new SerializationFactory(this.level);
		sfactory.write(o, eng);

		Assert.assertEquals(eng, eng.clone());
		Assert.assertEquals(eng, factory.read(new StringReader(w.toString())));
		Assert.assertEquals(eng, sfactory.read(new ByteArrayInputStream(o.toByteArray())));
	}

	private void reset(final Employee employee) {
		employee.setRepayment(null);
		employee.setRepaymentAmount(null);
		employee.setRepaymentType(null);
	}
}
